/*
TC2FRAMES converter - v2.0

Copyright © 2009, ogoon.fr. All rights reserved.
Graphics, HTML, CSS & JavaScript : Christophe Persoz (widget@ogoon.fr)

http://www.ogoon.fr/#widgets

----------------------------------------------------------------------------------------------

Creative Commons Licence
----------------------------------------------------------------------------------------------
This work is licensed under the Creative Commons Attribution-NonCommercial-No Derivative Works 2.0 License.
To view a copy of this license, visit http://creativecommons.org/licenses/by-nc-nd/3.0/
or seor send a letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.

*/

var current_version = "2.0";
var fps = 0;
var maxImg = 0;


//
// Function: load()
// Called by HTML body element's onload event when the widget is ready to start
//
function load()
{
    dashcode.setupParts();

    // Get the properties
    fps = parseFloat(document.getElementById('list-fps').object.getValue());
    maxImg = Math.round(23*3600*fps + (59*60)*fps + 59*fps + (fps-1));
    
    // Check for version
    document.getElementById('version-tooltip').innerText = getLocalizedString('VERSION') + current_version;
    document.getElementById('version-label-back').innerText = getLocalizedString('VERSION') + current_version;
    clean_for_new_version();
    setTimeout("checkVersion();", 4*1000);
}


//
// Function: showBack(event)
// Called when the info button is clicked to show the back of the widget
//
// event: onClick event from the info button
//
function showBack(event)
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget)
        widget.prepareForTransition("ToBack");
    
    document.getElementById('new_version-icon').style.display = 'none';

    front.style.display="none";
    back.style.display="block";
    
    if (window.widget)
        setTimeout('widget.performTransition();', 0);
}


//
// Function: showFront(event)
// Called when the done button is clicked from the back of the widget
//
// event: onClick event from the done button
//
function showFront(event)
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget)
        widget.prepareForTransition("ToFront");

    if (there_is_a_new_version)
        document.getElementById('new_version-icon').style.display = 'block';

    front.style.display="block";
    back.style.display="none";

    //startDisplayUpdateTimer();
    
    if (window.widget)
        setTimeout('widget.performTransition();', 0);
}


//
// Function: updateFrameRate(evt) 
// Set the new framerate used for conversions
//
// evt : Event onChange made on select framerate list
//
function updateFrameRate(evt)
{
    fps = parseFloat(this.getValue());
    maxImg = Math.round(23*3600*fps + (59*60)*fps + 59*fps + (fps-1));
    setEventLabel(getLocalizedString('NEW_FPS'),['%1'],[fps]);
    
    // reset all input values
    resetInput(document.getElementById('frames'));
    resetTC();
}


//
// Function: convertTC2Frm()
// Convert a TC to a number of frame with the current framerate
//
function convertTC2Frm()
{
   	var hh = Number(document.getElementById('tc-hh').value);
    var mm = Number(document.getElementById('tc-mm').value);
	var ss = Number(document.getElementById('tc-ss').value);
	var ff = Number(document.getElementById('tc-ff').value);
    
    if (hh<0 || mm<0 || ss<0 || ff<0)
    {
        setEventLabel(getLocalizedString('CANNOT_BE_NEG')); 
        return;
    }
    
    if (isNaN(hh) || isNaN(mm) || isNaN(ss) || isNaN(ff))
    {
        setEventLabel(getLocalizedString('MUST_BE_INT')); 
        return;
    }

	if (ff >= fps)
	{
		ss += getResult(ff, fps, true);
		ff = getRest(ff, fps, true);
	}

	if (ss >= 60)
	{
		mm += getResult(ss, 60, true);
		ss = getRest(ss, 60, true);
	}

	if (mm >= 60)
	{
		hh += getResult(mm, 60, true);
		mm = getRest(mm, 60, true);
	}

	if (hh >= 24)
	{
		hh = 0;
		mm = 0;
		ss = 0;
		ff = 0;
	} 

	var imgC = Math.round(hh*3600*fps) + Math.round(mm*60*fps) + Math.round(ss*fps) + ff;

	var img = imgC.toString();
	if (img.indexOf('.',0) != -1)
		imgNb = img.substr(0, img.indexOf('.',0));
	else
		imgNb = img;
 
    // show the result
	document.getElementById('frames').value = imgNb;
    //zeroFill(document.getElementById('frames'));
    
    // Copy value to clipboard
    if(window.widget)
    {
        widget.system("/usr/bin/osascript -e 'set the clipboard to \""+imgNb+"\"'", null).outputString;
        setEventLabel(getLocalizedString('FRM_TO_CLIPBOARD'));
    }
        
}


//
// Function: convertFrm2TC(imgNb)
// Return a TC equivalent to the number of frames provided
// The result is return as an interger, fps should be provided as a global var fps
//
// imgNb: Integer value to be translated
//
function convertFrm2TC(imgNb)
{
    var ffTc = '00';
	var ssTc = '00';
	var mmTc = '00';
	var hhTc = '00';
	var subsImg = 0;
	
    // Calcul HH
    var hhC = imgNb/(fps*3600);
    var hh = hhC.toString();
    if (hh.indexOf('.',0) != -1)
        hhTc = hh.substr(0, hh.indexOf('.',0));
    else
        hhTc = hh;
    
    if (hhTc < 10)
        hhTc = "0" + hhTc;
    subsImg = hhTc*(3600*fps);
    
    // Calcul MM
    var mmC = (imgNb - subsImg)/(60*fps);
    var mm = mmC.toString();
    if (mm.indexOf('.',0) != -1)
        mmTc = mm.substr(0, mm.indexOf('.',0));
    else
        mmTc = mm;
    
    if (mmTc < 10)
        mmTc = "0" + mmTc;
    subsImg += mmTc*(60*fps);
    
    //Calcul SS
    var ssC = (imgNb - subsImg)/fps;
    var ss = ssC.toString();
    if (ss.indexOf('.',0) != -1)
        ssTc = ss.substr(0, ss.indexOf('.',0));
    else
        ssTc = ss;
    
    if (ssTc < 10)
        ssTc = "0" + ssTc;
    subsImg += ssTc*fps;
    
    //Calcul FF
    var ffC = imgNb - subsImg;
    var ff = ffC.toString();
    if (ff.indexOf('.',0) != -1)
        ffTc = ff.substr(0, ff.indexOf('.',0));
    else
        ffTc = ff;
    
    if (ffTc < 10)
        ffTc = "0" + ffTc;

    /*			
    if (fps==23.98 || fps==29.97 || fps==59.94)
    {
        // Prise en compte du drop frame pour le NTSC 29.97 fps

        if (mmC%10 == 1)
        {
            // inclusion des frames 0 et 1
            if (ff.indexOf('.',0) != -1)
                ffTc = ff.substr(0, ff.indexOf('.',0));
            else
                ffTc = ff;
            
            if (ffTc < 10)
                ffTc = "0" + ffTc;
        } else {
            // exclusion des frames 0 et 1
            if (ffC > 2)
            {
                if (ff.indexOf('.',0) != -1)
                    ffTc = ff.substr(0, ff.indexOf('.',0));
                else
                    ffTc = ff;
                
                if (ffTc < 10)	
                ffTc = "0" + ffTc;
            } else
                ffTc = "00";
        }
    } else {
        if (ff.indexOf('.',0) != -1)
            ffTc = ff.substr(0, ff.indexOf('.',0));
        else
            ffTc = ff;
        
        if (ffTc < 10)
            ffTc = "0" + ffTc;
    }
    */
    
    // Update display
    document.getElementById('tc-hh').value = hhTc;
    document.getElementById('tc-mm').value = mmTc;
    document.getElementById('tc-ss').value = ssTc;
    document.getElementById('tc-ff').value = ffTc;
    
    // Copy value to clipboard
    var dataClipBoard = hhTc+":"+ mmTc+":"+ssTc+":"+ffTc;
    if (window.widget)
    {
        widget.system("/usr/bin/osascript -e 'set the clipboard to \""+dataClipBoard+"\"'", null).outputString;
        setEventLabel(getLocalizedString('TC_TO_CLIPBOARD'));
    }
    
    return;
}


//
// Function: getResult(Val, divBy, Int)
// Return the rest of Val divided by divBy.
// The result is return as an interger or float depending on Int is true or false
//
// Val: value to be divided
// divBy: the value used du divided Val
// Int: if true, the result will be returned as integer, and float is false
//
function getRest(Val, divBy, Int)
{
	if (divBy > 0)
	{
		if (Int)
			return (Val-(Math.round(Val/divBy)*divBy));
		else
			return (Val-((Val/divBy)*divBy));
	} else
		return (0);
}


//
// Function: getResult(Val, divBy, Int)
// Return the result of Val divided by divBy.
// The result is return as an interger or float depending on Int is true or false
//
// Val: value to be divided
// divBy: the value used du divided Val
// Int: if true, the result will be returned as integer, and float is false
//
function getResult(Val, divBy, Int)
{
	if (divBy > 0)
	{
		if (Int)
			return (Math.round(Val/divBy));
		else
			return (Val/divBy);
	} else
		return (0);
}


//
// Function: setEventLabel(label)
// Format and display the statut label
// You may need to customize this function if you remove or change the statut-label div
//
// label: Text to display for the label
// from: Array that will notice the replacement inside the label string (ie : %1, %2, etc.)
// by: Array that will take place instead of from
// from and by must be an array
//
function setEventLabel(label, from, by)
{
    var eventLabelElement = document.getElementById("statut-label");
    if (eventLabelElement)
    {
        if ( from != undefined && by != undefined)
        {
            var s = new String(label);
            for (var n=0; n<from.length; n++)
                s = s.replace(from[n], by[n]);
            eventLabelElement.innerText = s;
        } else
            eventLabelElement.innerText = label;
    }
}


//
// Function: clearInput(obj) 
// Empty input field if null or initialized
//
// obj : DOM object, must be an input tag
//
function clearInput(obj)
{
    if (obj.maxLength == 2)
        var reg = /[0-9]{1,}/;
    else if (obj.maxLength == 7)
        var reg = /[0-9]{1,}/;
    else
        return;
    
    if (reg.test(obj.value))
        obj.value = '';
}


//
// Function: resetInput(obj) 
// Fill with 0 the input field if empty or null
//
// obj : DOM object, must be an input tag
//
function resetInput(obj)
{
    if (obj.maxLength == undefined)
        return;
        
    var s = new String();
    for (i=0;i<obj.maxLength;i++)
        s += '0';
    obj.value = s;
}


//
// Function: resetTC(obj) 
// Reset TCFields;
//
// obj : DOM text or inputfield object
//
function resetTC()
{
    resetInput(document.getElementById('tc-hh'));
    resetInput(document.getElementById('tc-mm'));
    resetInput(document.getElementById('tc-ss'));
    resetInput(document.getElementById('tc-ff'));
}


//
// Function: zeroFill(obj) 
// Complete with 0 the input value until maxlength
//
// obj : DOM text or inputfield object
//
function zeroFill(obj)
{
    if (obj.maxLength == undefined || obj.maxLength == obj.value.length)
        return;

    var s = new String();
    for (i=0;i<(obj.maxLength-obj.value.length);i++)
        s += '0';
    
    obj.value = s + obj.value;
}


//
// Function: updateFrm(event,obj) 
// Update the Frame number input field accordingly to the TC provided by all TC inputs
//
// event: onKeyPressed event from the obj input field
// obj : DOM text or inputfield object
//
function updateFrm(event, obj)
{
    if (event.keyCode != 9 || event.keyCode != 25) // tab or shift-tab
    {
        // clear statut
        setEventLabel('');
        
        // check values
        var tcVal = parseInt(lTrim(obj.value,"0"));
        switch(obj.id)
        {
            case 'tc-hh':
                var maxValue = 23;
                if (tcVal > 23 || tcVal < 0)
                {
                    setEventLabel(getLocalizedString('BAD_TCHH'));
                    resetInput(obj); 
                }
            break;
            case 'tc-mm':
                var maxValue = 59;
                if (tcVal > 59 || tcVal < 0)
                {
                    setEventLabel(getLocalizedString('BAD_TCMM'));
                    resetInput(obj); 
                }
            break;
            case 'tc-ss':
                var maxValue = 59;
                if (tcVal > 59 || tcVal < 0)
                {
                    setEventLabel(getLocalizedString('BAD_TCSS'));
                    resetInput(obj); 
                }
            break;
            case 'tc-ff':
                var maxValue = fps-1;
                if (tcVal > maxValue || tcVal < 0)
                {
                    setEventLabel(getLocalizedString('BAD_TCFF'),['%1'],[Math.round(fps-1)]);
                    resetInput(obj); 
                }
            break;
            default: 
                return;
        }
        // calculate TC
        convertTC2Frm();
    }
}


// Function: leaveTC(event, obj)
function leaveTC(event,obj)
{
    if (isNaN(parseInt(lTrim(obj.value,"0")))) // tab
        resetInput(obj);
    else
        zeroFill(obj);
 
    updateFrm(event,obj);
}


//
// Function: updateTC(event,obj) 
// Update the TC input field accordingly to the frame number provided by obj input
//
// event: onKeyPressed event from the obj input field
// obj : DOM text or inputfield object
//
function updateTC(event, obj)
{
    var imgNb = parseInt(lTrim(obj.value,"0"));
    
    // clear statut
    setEventLabel('');
    
    // check datas input
    if (isNaN(imgNb))
    {
        setEventLabel(getLocalizedString('MUST_BE_INT'));
       // clearInput(obj);
        return;
    }
    if (imgNb<0)
    {
        setEventLabel(getLocalizedString('CANNOT_BE_NEG'));
        clearInput(obj);
        return;
    }
	if (imgNb > maxImg)
	{
        setEventLabel(getLocalizedString('BAD_FRAMES_NB'),['%1','%2'],[maxImg, fps]);
        clearInput(obj);
        resetTC();
		return;
	}
    // calculate TC
    convertFrm2TC(imgNb);
}


//
// Function: lTrim(str, charlist) 
// trim any char, provided on charlist, on left side of the string str
//
// str: String to be trim
// charlist : regex charlist used to trim str
//
function lTrim(str,charlist)
{
    var re = new RegExp('^[' + charlist + ']+', 'g');
    return (str+'').replace(re, '');
}


//
// Function: numericsOnly(evt) 
// Autorize only numeric keyCode as entry
//
// evt : any keyPressed to be restricted
//
function numericsOnly(evt)
{
    // Insérer le code ici
    if (evt.metaKey) {// command key down
    	return true;// process for meta key being down
	}
    
    evt = (evt) ? evt : event;
    var charCode = (evt.charCode) ? evt.charCode : ((evt.keyCode) ? evt.keyCode : ((evt.which) ? evt.which : 0));

    if (!((charCode < 32 ) || (charCode > 47 && charCode < 58) || (charCode == 63272) || ((charCode >= 63232) && (charCode <= 63235)))) {
        return (false);
    } else if (charCode == 13 || charCode == 3) { // enter keys
		return (false);
    }
    
    return (true);
}


//
// Function: gotoWeb(event) 
// Link to OGOON media website
//
// event: onClick event provided on click on link
//
function gotoWeb(event)
{
    websiteURL = "http://www.ogoon.fr/#widgets";
    widget.openURL(websiteURL);
}


//
// Function: gotoWeb(event) 
// Link to OGOON media website
//
// event: onClick event provided on click on link
//
function downloadNewVersion(event)
{
    websiteURL = "http://www.ogoon.fr/_widget/tc2frames/tc2frames.zip";
    widget.openURL(websiteURL);
}


// Initialize the Dashboard event handlers
if (window.widget) {
   // widget.onhide = hide;
   // widget.onshow = show;
}


/*
    Version Check
*/
var version_req_stop_timeout = false;
var version_req_timeout = false;
var there_is_a_new_version = false;


function clean_for_new_version()
{
    try {
		version_value = widget.preferenceForKey ("tc2frames_version");
		if (current_version != version_value)
        {
            cleaner();
			widget.setPreferenceForKey(current_version, "tc2frames_version");
		}
	}
	catch (ex)
    {
        cleaner();
		widget.setPreferenceForKey(current_version, "tc2frames_version");
	}
}


function checkVersion()
{
	loadVersionPage('http://www.ogoon.fr/_widget/tc2frames/version.xml');
}


function loadVersionPage(url)
{
	if (window.XMLHttpRequest) {
		version_req = new XMLHttpRequest();
		version_req.overrideMimeType("text/xml");
		version_req.open("GET", url, true);
		version_req.setRequestHeader("Cache-Control", "no-cache");
		version_req.onreadystatechange = parseVersionPage;
		version_req.send(null);
		
		version_req_stop_timeout = false;
		version_req_timeout = false;
		setTimeout("version_req_send_timeout();",5000);
	}
}

function version_req_send_timeout()
{
	if (version_req_stop_timeout)
        return;

	version_req_timeout = true;
}

function isGreaterVersion(r1, r2)
{
	r1_numbers = r1.split('.');
	r2_numbers = r2.split('.');
	
	r1_value = 0;
	r2_value = 0;
	
	for (i=0;i<r1_numbers.length;i++)
		r1_value += r1_numbers[i]*(Math.pow(10,-i*3));
	
	for (i=0;i<r2_numbers.length;i++)
		r2_value += r2_numbers[i]*(Math.pow(10,-i*3));
	
	if (r1_value > r2_value)
        return (true);
	
	return (false);
}

function parseVersionPage()
{
	if (version_req_timeout)
    {
        version_req.abort();
        version_req_timeout = false;
        version_req_stop_timeout = true;
        return;
    }
	
	if ((version_req.readyState == 4) && (version_req.status == 200))
    {
		version_req_stop_timeout = true;
		version_req_timeout = false;
		
		latest_version = version_req.responseXML.getElementsByTagName("version");
		latest_version = latest_version[0].firstChild.data;
	
		// version comparison
		if (isGreaterVersion(latest_version,current_version))
			there_is_a_new_version = true;
 
		if (there_is_a_new_version)
        {
			document.getElementById('new_version-icon').style.display = "block";
			tooltip = getLocalizedString('NEW_VERSION_AVAILABLE') + latest_version + " (" + getLocalizedString('YOUR_CURRENTLY_HAVE') + current_version + ")";
            document.getElementById('version-tooltip').innerText = tooltip;
			document.getElementById('new_version-icon').setAttribute ("title", tooltip);
		} else {
            document.getElementById('new_version-icon').style.display = "none";
        }
	}
}

function cleaner()
{
	widget.setPreferenceForKey("", "tc2frames_version");
}